==> shrimper-0.2.0.tar.gz: shrimper-0.2.0/shrimper/fingerling.py <==
#!/usr/bin/env python
"""
Fingerling

Born to eat xiami.com
"""
import re
import os
import sys
import subprocess
import xml.etree.ElementTree as ET
import urllib.request
import urllib.parse
import html

from mutagen.mp3 import MP3
import mutagen.id3 as ID3


def gen_tracklist_urls(url_type, id):
    """Calc the url depending the ids of songs, albums or omnibuses."""
    url_base = 'http://www.xiami.com/song/playlist/id/{0}'
    url_single = url_base + '/object_name/default/object_id/0'
    url_album = url_base + '/type/1'
    url_omnibus = url_base + '/type/3'

    if url_type == 'single':
        template = url_single
    elif url_type == 'album':
        template = url_album
    elif url_type == 'omnibus':
        template = url_omnibus
    else:
        pass # do something :^)

    return template.format(id)


def get_tracklist_from_url(url):
    """Get tracklist from url"""
    raw = get_response(url)
    xml_str =  raw.decode('utf-8')
    # remove XML namespace
    xml_str = re.sub(r'xmlns=\".*?\"', '', xml_str)
    element_tree = ET.fromstring(xml_str)
    tracklist = []

    for item in element_tree.iter('track'):
        track = {}
        for key in ['title', 'album_name', 'artist', 'location', 'lyric', 'pic']:
            value = item.find(key).text
            if value:
                if key == 'title' or key == 'album_name':
                    # \ and / bring fliesystem error, so replace them with _
                    track[key] = re.sub(r'[\\/]', '_',
                                        html.unescape(value))
                else:
                    track[key] = html.unescape(value)
            else:
                track[key] = None
        tracklist.append(track)
    return tracklist


def decrypt_location(location):
    """
    Analyze the track's location
    """
    url = location[1:]
    urllen = len(url)
    row_number = int(location[:1])

    col_number = urllen // row_number  # basic column count: 34
    spare_char = urllen % row_number   # count of rows that have 1 more column

    matrix = []
    for i in range(row_number):
        length = col_number + 1 if i < spare_char else col_number

        matrix.append(url[:length])
        url = url[length:]

    url = ''
    for i in range(urllen):
        url += matrix[i % row_number][i // row_number]

    return urllib.parse.unquote(url).replace('^', '0')


def get_response(url):
    """
    Abtain the xml structure in byte string.
    If you push a request without the headers, there may trigger 503/403 error.
    """
    headers = {
        'User-Agent': 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)',
        'Referer': 'http://www.xiami.com/song/play'
    }

    request = urllib.request.Request(url, headers=headers)

    try:
        response = urllib.request.urlopen(request)
        return response.read()
    except urllib.URLError as e:
        print(e)
        return ''


def gen_album_image_url(basic, size=None):
    """ Get album image url in a specific size:
    * Leave None for the largest
    * 4 for a reasonable size
    """
    if size:
        rep = r'\1_%d\2' % size
    else:
        rep = r'\1\2'
    return re.sub(r'^(.+)_\d(\..+)$', rep, basic)


def add_id3_tag(filename, track):
    """Write id3v2.4 tag to mp3 file."""
    f = MP3(filename)
    f.add_tags()

    # Lyric
    if track['lyric']:
        lyric = get_response(track['lyric'])
        f.tags.add(ID3.USLT(
            encoding = 3,
            desc = 'Lyrics',
            text = lyric.decode('utf-8')
        ))

    # Track Name
    f.tags.add(ID3.TIT2(
        encoding = 3,
        text = track['title']
    ))

    # Album Name
    f.tags.add(ID3.TALB(
        encoding = 3,
        text = track['album_name']
    ))

    # Lead Artist/Performer/Soloist/Group
    f.tags.add(ID3.TPE1(
        encoding = 3,
        text = track['artist']
    ))

    # Album Cover
    image = get_response(gen_album_image_url(track['pic'], 4))
    if image:
        f.tags.add(ID3.APIC(
            encoding = 3,
            mime = 'image/jpeg',
            type = 3,  # album front cover
            desc = 'Cover',
            data = image
        ))

    # tag function test: print tag info
    #sys.stdout.write(f.pprint() + '\n')
    f.save()

==> shrimper-0.2.1.tar.gz: shrimper-0.2.1/shrimper/fingerling.py <==
#!/usr/bin/env python
"""
Fingerling

Born to eat xiami.com
"""
import re
import os
import sys
import subprocess
import xml.etree.ElementTree as ET
import urllib.request
import urllib.parse
import html

from mutagen.mp3 import MP3
import mutagen.id3 as ID3


def gen_tracklist_urls(url_type, id):
    """Calc the url depending the ids of songs, albums or omnibuses."""
    url_base = 'http://www.xiami.com/song/playlist/id/{0}'
    url_single = url_base + '/object_name/default/object_id/0'
    url_album = url_base + '/type/1'
    url_omnibus = url_base + '/type/3'

    if url_type == 'single':
        template = url_single
    elif url_type == 'album':
        template = url_album
    elif url_type == 'omnibus':
        template = url_omnibus
    else:
        pass # do something :^)

    return template.format(id)


def get_response(url):
    """
    Abtain the xml structure in byte string.
    If you push a request without the headers, there may trigger 503/403 error.
    """
    headers = {
        'User-Agent': 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 7.1; Trident/5.0)',
        'Referer': 'http://www.xiami.com/song/play'
    }
    request = urllib.request.Request(url, headers=headers)

    try:
        response = urllib.request.urlopen(request)
        return response.read()
    except urllib.request.URLError:
        return None


def get_tracklist_from_url(url):
    """Get tracklist from url"""
    raw = get_response(url)
    if raw == b'':
        sys.stdout.write('ERRROR: Nothing in this URL: {}\n'.format(url))
        exit(1)
    xml_str =  raw.decode('utf-8')
    # remove XML namespace
    xml_str = re.sub(r'xmlns=\".*?\"', '', xml_str)
    element_tree = ET.fromstring(xml_str)
    tracklist = []

    for item in element_tree.iter('track'):
        track = {}
        for key in ['title', 'album_name', 'artist', 'location', 'pic']:
            value = item.find(key).text
            if value:
                if key == 'title' or key == 'album_name':
                    # \ and / bring fliesystem error, so replace them with _
                    track[key] = re.sub(r'[\\/]', '_',
                                        html.unescape(value))
                else:
                    track[key] = html.unescape(value)
            else:
                track[key] = None
        tracklist.append(track)
    return tracklist


def decrypt_location(location):
    """
    Analyze the track's location
    """
    url = location[1:]
    urllen = len(url)
    row_number = int(location[:1])

    col_number = urllen // row_number  # basic column count: 34
    spare_char = urllen % row_number   # count of rows that have 1 more column

    matrix = []
    for i in range(row_number):
        length = col_number + 1 if i < spare_char else col_number

        matrix.append(url[:length])
        url = url[length:]

    url = ''
    for i in range(urllen):
        url += matrix[i % row_number][i // row_number]

    return urllib.parse.unquote(url).replace('^', '0')


def gen_album_image_url(basic, size=None):
    """ Get album image url in a specific size:
    * Leave None for the largest
    * 4 for a reasonable size
    """
    if size:
        rep = r'\1_%d\2' % size
    else:
        rep = r'\1\2'
    return re.sub(r'^(.+)_\d(\..+)$', rep, basic)


def add_id3_tag(filename, track):
    """Write id3v2.4 tag to mp3 file."""
    f = MP3(filename)
    f.add_tags()

    # Track Name
    f.tags.add(ID3.TIT2(
        encoding = 3,
        text = track['title']
    ))

    # Album Name
    f.tags.add(ID3.TALB(
        encoding = 3,
        text = track['album_name']
    ))

    # Lead Artist/Performer/Soloist/Group
    f.tags.add(ID3.TPE1(
        encoding = 3,
        text = track['artist']
    ))

    # Album Cover
    image = get_response(gen_album_image_url(track['pic'], 4))
    if image:
        f.tags.add(ID3.APIC(
            encoding = 3,
            mime = 'image/jpeg',
            type = 3,  # album front cover
            desc = 'Cover',
            data = image
        ))

    # tag function test: print tag info
    #sys.stdout.write(f.pprint() + '\n')
    f.save()
